# -*- coding: utf-8 -*-
"""
Construction d'une base Communes-Régions-Départements
à partir de l'API https://geo.api.gouv.fr/
"""
import urllib.request
import sqlite3
import json



class Georequest:
    """
    Classe créant des objets réponses à une requête sur l'API
    de https://geo.api.gouv.fr/
    """
    
    def __init__(self, url_requete):
        """
        Trois attributs d'instance :
            self.url_requete : url de la requête
            self.reponse : réponse HTTP conforme à l'API
            self.json_dico : conversion en dictionnaire Python du contenu de la réponse au format json
        """
        self.url_requete = url_requete
        self.reponse = urllib.request.urlopen(self.url_requete).read()
        self.json_dico = json.loads(self.reponse.decode())
        
    def get_json(self):
        """
        Getter de l'attribut self.json_dico, renvoie sa valeur
        """
        return self.json_dico
        
     
class Base:
    """
    Classe créant un objet base de données sqlite3 avec ses principaux
    attibuts (connecteur, curseur) et méthodes d'accès (création de table, requete)
    """
    
    def __init__(self, chemin):
        """
        Deux attributs d'instance (voir la doc https://docs.python.org/3.10/library/sqlite3.html)
            self.connecteur : connexion à la base de données (client)
            self.curseur : curseur de parcours de la base
        """
        self.connecteur = sqlite3.connect(chemin)
        self.curseur = self.connecteur.cursor()
        
    def activate_foreign_key(self):
        """Activation des contraintes d'intégrité référentielle
        des clefs étrangères (désactivé par défaut avec sqlite)"""
        self.connecteur.execute("PRAGMA foreign_keys = ON;")
    
    def create_table(self, requete_creation):
        """Exécute une requête de création de table"""
        self.curseur.execute(requete_creation)
        
    def execute_request(self, requete, valeurs=()):
        """
        Exécute une requête avec un tuple de valeurs éventuellement vide
        Voir https://docs.python.org/3/library/sqlite3.html#sqlite3-howtos
        pour l'utilisation des placeholders afin de prévenir les injections 
        de code SQL dans les requêtes
        """
        rep = self.curseur.execute(requete, valeurs)
        return rep.fetchall()
        
    def commit(self):
        """Enrgistrement des modifications sur la base"""
        self.connecteur.commit()
        
    def close(self):
        """Fermeture de la connexion à la base"""
        self.connecteur.close()

