# -*- coding: utf-8 -*-
"""
Client pour le module geo.py
"""
import geo


def creer_base():
    """création de la base, renvoie l'objet créé avec la classe geo.Base"""
    base = geo.Base("communes-regions-departements.db")
    # activation des clefs étrangères
    base.activate_foreign_key()
    return base


def requete_web_api():
    """
    récupération des données par requete sure l'API "https://geo.api.gouv.fr
    """
    regions_data = geo.Georequest("https://geo.api.gouv.fr/regions/")
    departements_data = geo.Georequest("https://geo.api.gouv.fr/departements/")
    communes_data = geo.Georequest("https://geo.api.gouv.fr/communes/")
    return {
        "regions": regions_data,
        "departements": departements_data,
        "communes": communes_data,
    }


def creer_tables(base):
    """
    Création des tables (schémas relationnels)
    Modifie l'objet base par effet de bord
    """
    # création de la table régions
    creation_table_regions = (
        """CREATE TABLE regions (code TEXT PRIMARY KEY, nom TEXT);"""
    )
    base.create_table(creation_table_regions)
    # création de la table départements
    creation_table_departements = """CREATE TABLE departements (code TEXT PRIMARY KEY, nom TEXT, codeRegion TEXT,
                                                                FOREIGN KEY (codeRegion) REFERENCES regions(code));"""
    base.create_table(creation_table_departements)
    # =============================================================================
    #     # création de la table communes à compléter
    # =============================================================================
    creation_table_communes = """a completer"""
    # décommentez la ligne en  dessous
    base.create_table(creation_table_communes)
    # on enregistre les changements
    base.commit()


def peupler_tables(base, data):
    """
    Peuplement des tables à partir des données récupérées depuis l'API Web
    """
    # D'abord les régions
    for region in data["regions"].get_json():
        requete_insertion = """INSERT INTO regions VALUES(?, ?);"""
        valeurs = (region["code"], region["nom"])
        base.execute_request(requete_insertion, valeurs)
    # Puis les départements
    liste_code_departements = []
    for departement in data["departements"].get_json():
        if departement["code"] not in liste_code_departements:
            liste_code_departements.append(departement["code"])
        requete_insertion = """INSERT INTO departements VALUES(?, ?, ?);"""
        valeurs = (
            departement["code"],
            departement["nom"],
            departement["codeRegion"],
        )
        base.execute_request(requete_insertion, valeurs)
    # et enfin les communes
    for commune in data["communes"].get_json():
        print(commune)
        if "population" not in commune:
            commune["population"] = 0
        # =============================================================================
        ## insérer en dessous une ligne de code permettant de ne pas insérer les valeurs
        # de  communes qui n'ont pas de code département et qui rendent la base incohérente
        # =============================================================================
        # compléter les pointillés ci-dessous
        if commune["codeDepartement"] in ...:
            requete_insertion = """INSERT INTO communes VALUES(?, ?, ?, ?);"""
            valeurs = (
                commune["code"],
                commune["nom"],
                int(commune["population"]),
                commune["codeDepartement"],
            )
            base.execute_request(requete_insertion, valeurs)
    # on enregistre les changements
    base.commit()


def requetes_base(base):
    """Placer ici quelques requêtes sur la base"""
    # quelques requêtes sur la base
    print("Nombre de régions")
    rep = base.execute_request("SELECT COUNT(*) FROM regions;")
    print(rep)
    print("Départements dans la région AURA (code 84)")
    rep = base.execute_request(
        """SELECT departements.nom 
                                   FROM departements JOIN regions ON departements.codeRegion = regions.code 
                                   WHERE regions.code = 84;"""
    )
    print(rep)


def main():
    """Programme principal"""
    base_crd = creer_base()
    creer_tables(base_crd)
    data_crd = requete_web_api()
    peupler_tables(base_crd, data_crd)
    requetes_base(base_crd)
    # on ferme la connexion à la base
    base_crd.close()


if __name__ == "__main__":
    main()
